import de.voidplus.leapmotion.*; //BRING IN THE LEAP
import oscP5.*; //library to receive data via OSC from the other sketch
import netP5.*;

LeapMotion leap;

OscP5 oscP5;

NetAddress procLocation;
NetAddress maxLocation;


int num_elements = 14488; //lines in the CSV file

float megarows, megacolums; //rows and columns of the matrix

int g_index; //some sort of global index for all elements, used in setup()

//random note: USE ScreenX and ScreenY to get 2D coords of a 3D element if needed - no more needed

float[] xs = new float[num_elements];
float[] ys = new float[num_elements];
float[] zs = new float[num_elements];
float[] sizes = new float[num_elements];
color[] colors = new color[num_elements];
int[] shape_id = new int[num_elements];
String[] names = new String[num_elements];

String[][] tab = new String[16][num_elements];

float minX, maxX, minY, maxY, minZ, maxZ;

int sMammals, sReptiles, sAmphibes, sFishes, sBirds;

PVector neutralHandPosition; //Vector to store neutral hand position

LowPass lpx, lpy, lpz; // creating a filter for each coordinate
int LPSmoothLevel = 5; // smooth level for lowpass filter

boolean isPositioned = false;


void setup() {
  size(1430, 1006, P3D);
  //frame.setResizable(true);
  
  oscP5 = new OscP5(this,5005);
  procLocation = new NetAddress("127.0.0.1", 5005);
  maxLocation = new NetAddress("127.0.0.1", 5002);
  
  background(0);
  smooth(2);
  rectMode(CENTER);
  noStroke();
  
  tab = importCSV(); //importing my csv
  //println(tab[250][12]); //just checking if correctly imported

  megarows = megacolums = int(sqrt(num_elements));

  for (int j = 0; j < megarows; j++) { //generating coordinates for matrix
    for (int k = 0; k < megacolums; k++) {
      g_index++;
      xs[g_index] = ((k/megacolums) * 8000 - 4000) + random(-2, 2);
      ys[g_index] = ((j/megarows) * 8000 - 4000) + random(-2, 2);
      //println(i, xs[i], ys[i]);
    }
  }

  while (g_index<num_elements) { //..even for the elements exceeding sqrt(num_elements)
    xs[g_index]= random(-4000, 4000);
    ys[g_index]= random(-4000, 4000);
    g_index++;
  }

  for (int i = 0; i<num_elements; i++) {

    //placing elements according to risk level
    if (tab[i][11].equals("EW")) {
      zs[i] = random(6000, 7000);
      //println("CR");
    }
    if (tab[i][11].equals("CR")) {
      zs[i] = random(7000, 7500);
    } else if (tab[i][11].equals("EN")) {
      zs[i] = random(5500, 6000);
    } else if (tab[i][11].equals("VU")) {
      zs[i] = random(4000, 4500);
    } else if (tab[i][11].equals("LR/nt")) {
      zs[i] = random(2500, 3000);
    } else if (tab[i][11].equals("NT")) {
      zs[i] = random(1000, 1500);
    } else if (tab[i][11].equals("DD")) {
      zs[i] = random(300, 500);
    } else {
      zs[i] = random(300, 500);
    }

    //assigning size and color to each element
    sizes[i]=random(40, 50);  


    //assigning shape and color to each element
    // mammals sqaure = 0, birds (aves) rombo = 1, amphibia rev triangle = 2, 
    //fishes triangle up = 3, reptiles circle = 4 

    if (tab[i][2].equals("MAMMALIA")) {
      shape_id[i] = 0; 
      colors[i] = color(0, 129, 227);
    } else if (tab[i][2].equals("AVES")) { //birds
      shape_id[i] = 1;
      colors[i] = color(102, 36, 131);
    } else if (tab[i][2].equals("AMPHIBIA")) {
      shape_id[i] = 2;
      colors[i] = color(255, 167, 0);
    } else if ((tab[i][2].equals("CHONDRICHTHYES")) || (tab[i][2].equals("ACTINOPTERYGII")) || (tab[i][2].equals("SARCOPTERYGII")) || (tab[i][2].equals("AGNATHA")) || (tab[i][2].equals("CEPHALASPIDOMORPHI")) || (tab[i][2].equals("MYXINI"))) {//fishes
      shape_id[i]=3;
      colors[i] = color(230, 0, 126);
    } else if (tab[i][2].equals("REPTILIA")) {
      shape_id[i]=4;
      colors[i] = color(101, 179, 46);
    }
    
    names[i] = (tab[i][5]+" "+tab[i][6]);
    textMode(MODEL);
    textSize(120);
    
  }

  //initializing camera 
  camera(0, 0, -1000, 0, 0, 5000, 0, 1, 0);
  // old position: 0, 0, -1000, 0, 0, 5000, 0, 1, 0
  
  minX = -1000; //magic cube where the names are shown
  maxX = 1000;
  minY = -1000;
  maxY = 1000;
  minZ = -900;
  maxZ = -200;

  //initializing LeapMotion
  leap = new LeapMotion(this);

  //setting the neutral hand position
  neutralHandPosition = new PVector(670, 700, 42);

  //initializing lowpass filter
  lpx = new LowPass(LPSmoothLevel); //argument is queue depth
  lpy = new LowPass(LPSmoothLevel);
  lpz = new LowPass(LPSmoothLevel);

  //filling the filter with the neutral position
  for (int riempifiltro = 0; riempifiltro < LPSmoothLevel; riempifiltro++) {
    lpx.input(neutralHandPosition.x);
    lpy.input(neutralHandPosition.y);
    lpz.input(neutralHandPosition.z);
  }
}


void draw() {
  //background(random(255), random(255), random(255)); 
  background(0);
  
  if (!isPositioned) {
   frame.setLocation(250, 0);
   isPositioned = true;
  }
  
  //drawing elements

  for ( int i = 0; i<num_elements; i++) {
    translate(xs[i], ys[i], zs[i]);
    fill(colors[i]);

    // assigning a shape to each element
    // mammals sqaure = 0, birds (aves) rombo = 1, amphibia rev triangle = 2, 
    // fishes triangle up = 3, reptiles circle = 4 

    if (shape_id[i] == 0) { //mammals, square
      rect(0, 0, sizes[i], sizes[i]);
    } else if (shape_id[i] == 1) { //birds, rombo
      rotate(radians(45));
      rect(0, 0, sizes[i], sizes[i]); 
      rotate(radians(-45));
    } else if (shape_id[i] == 2) { //amphibia, rev triangle
      triangle(-sizes[i]/2, -sizes[i]/2, sizes[i]/2, -sizes[i]/2, 0, sizes[i]/3);
    } else if (shape_id[i] == 3) { //fishes, triangle up
      triangle(sizes[i]/2, sizes[i]/2, -sizes[i]/2, sizes[i]/2, 0, -sizes[i]/3);
    } else if (shape_id[i] == 4) { //reptiles, circle
      ellipse(0, 0, sizes[i], sizes[i]);
    }
    
    if ((xs[i] > minX) && (xs[i] < maxX) && 
        (ys[i] > minY) && (ys[i] < maxY) &&
        (zs[i] > minZ) && (zs[i] < maxZ)){
          fill (255);
          pushMatrix();
          scale(-0.1, 0.1);
          text(names[i], 0, 0);
          popMatrix();
          
          if (shape_id[i] == 0){
            sMammals++;
          }
          else if (shape_id[i] == 1){
            sBirds++;
          }
          else if (shape_id[i] == 2){
            sAmphibes++;
          }
          else if (shape_id[i] == 3){
            sFishes++;
          }
          else if (shape_id[i] == 4){
            sReptiles++;
          }
                                
      }

      translate(-xs[i], -ys[i], -zs[i]);
  }
  
  OscMessage leapMessage = new OscMessage("/animals"); //sending animals to sonify via OSC
  leapMessage.add(sMammals);
  leapMessage.add(sBirds);
  leapMessage.add(sAmphibes);
  leapMessage.add(sFishes);
  leapMessage.add(sReptiles);
  
  oscP5.send(leapMessage, maxLocation); 
  
  sMammals = 0;
  sBirds = 0;
  sAmphibes = 0;
  sFishes = 0;
  sReptiles = 0;

  Hand l_hand = leap.getLeftHand();

  //println("pos:", l_hand.getPosition());
  //println("angles:", l_hand.getRoll(), l_hand.getPitch(), l_hand.getYaw());
  //println(l_hand.getFingers().size());

  if (l_hand.getSphereRadius() > 60) {
    lpx.input(l_hand.getPosition().x);
    lpy.input(l_hand.getPosition().y);
    lpz.input(l_hand.getPosition().z);

    beginCamera();
    translate((lpx.output - neutralHandPosition.x)/10, 
              -(lpy.output - neutralHandPosition.y)/10, 
              -(lpz.output - neutralHandPosition.z));
    endCamera();
    
    minX = minX - (lpx.output - neutralHandPosition.x)/10;
    maxX = maxX - (lpx.output - neutralHandPosition.x)/10;
    minY = minY + (lpy.output - neutralHandPosition.y)/10;
    maxY = maxY + (lpy.output - neutralHandPosition.y)/10;
    minZ = minZ + (lpz.output - neutralHandPosition.z);
    maxZ = maxZ + (lpz.output - neutralHandPosition.z);
    
    OscMessage leapCoords = new OscMessage("/leapCoords"); //sending leap coords via OSC
    leapCoords.add(lpx.output - neutralHandPosition.x);
    leapCoords.add(lpy.output - neutralHandPosition.y);
    leapCoords.add(lpz.output - neutralHandPosition.z);
    //println("OSC INVIO OSC INVIO LEAP!");
    oscP5.send(leapCoords, procLocation); 
  }
 
}


//change zoom level with keyboard
void keyPressed() {
/*
  if (key == 'b') {
    beginCamera();
    translate(0, 0, 30);
    endCamera();
    
  }

  if (key == 'v') {
    beginCamera();
    translate(0, 0, -30);
    endCamera();
  }
  */
  if (key == ' ') { //bringing camera back to def position when spacebar is pressed
    camera(0, 0, -1000, 0, 0, 5000, 0, 1, 0);
    
    OscMessage leapMessage = new OscMessage("/reset"); //sending leap coords via OSC
    
    oscP5.send(leapMessage, procLocation); 
    
    minX = -1000;
    maxX = 1000;
    minY = -1000;
    maxY = 1000;
    minZ = -900;
    maxZ = -200;
  }

}

//function to import a .csv file
String[][] importCSV() {

  String lines[] = loadStrings("supertable.csv");
  String [][] csv;
  int csvWidth=0;

  //calculate max width of csv file
  for (int i=0; i < lines.length; i++) {
    String [] chars=split(lines[i], ';'); //CHECK DAT CHARACTER...
    if (chars.length>csvWidth) {
      csvWidth=chars.length;
    }
  }

  //create csv array based on # of rows and columns in csv file
  csv = new String [lines.length][csvWidth];

  //parse values into 2d array
  for (int i=0; i < lines.length; i++) {
    String [] temp = new String [lines.length];
    temp= split(lines[i], ';'); //...AND DAT TOO
    for (int j=0; j < temp.length; j++) {
      csv[i][j]=temp[j];
    }
  }
  return csv;
}


class LowPass { //lowpass filter for the LM input
  ArrayList buffer;
  int len;
  float output;

  LowPass(int len) {
    this.len = len;
    buffer = new ArrayList(len);
    for (int i = 0; i < len; i++) {
      buffer.add(new Float(0.0));
    }
  }

  void input(float v) {
    buffer.add(new Float(v));
    buffer.remove(0);

    float sum = 0;
    for (int i=0; i<buffer.size (); i++) {
      Float fv = (Float)buffer.get(i);
      sum += fv.floatValue();
    }
    output = sum / buffer.size();
  }
}
